﻿using neo365.Server.Http;
using System;
using System.Threading;
using System.Windows.Forms;

namespace SimpleHttpServer
{
    public partial class WebDaemon : Form
    {
        private Thread DaemonThread;
        private ServerProcess httpServer;

        public bool IsServerStart
        {
            set
            {
                btnStart.Enabled = !value;
                btnStop.Enabled = !btnStart.Enabled;
            }
            get
            {
                return btnStart.Enabled;
            }
        }

        public WebDaemon()
        {
            InitializeComponent();
            this.FormClosing += WebDaemon_FormClosing;

            btnStart.Enabled = true;
            btnStop.Enabled = !btnStart.Enabled;

            httpServer = new ServerProcess(this, 80);
            httpServer.Reqeust += HttpServer_Reqeust;

            IsServerStart = false;
        }

        private void HttpServer_Reqeust(object sender, HttpRequestEventArgs e)
        {
            txtLog.AppendText(String.Format("[{0}] Request!\r\nHost: {1} \r\n", DateTime.Now.ToString("HH:mm:ss"), e.Request.Headers["Host"]));
            txtLog.AppendText(String.Format("URL: {0} \r\n", e.Request.Url));
            txtLog.AppendText(String.Format("User-Agent: {0} \r\n", e.Request.Headers["User-Agent"]));

            if (e.Request.GetParameters.Count > 0)
            {
                txtLog.Text += "[GET]\r\n";
                foreach (string key in e.Request.GetParameters.AllKeys)
                {
                    txtLog.AppendText(String.Format("KEY: {0} | VALUE: {1}\r\n", key, e.Request.GetParameters[key]));
                }
            }
            if (e.Request.PostParameters.Count > 0)
            {
                txtLog.Text += "[POST]\r\n";
                foreach (string key in e.Request.PostParameters.AllKeys)
                {
                    txtLog.AppendText(String.Format("KEY: {0} | VALUE: {1}\r\n", key, e.Request.PostParameters[key]));
                }
            }
            if (string.IsNullOrEmpty(e.Request.json) == false)
            {
                txtLog.Text += "[JSON]\r\n" + e.Request.json + "\r\n";
            }
        }

        private void WebDaemon_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (DaemonThread != null)
            {
                if (DaemonThread.IsAlive)
                {
                    httpServer.Stop();
                }

                DaemonThread.Abort();
            }
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            DaemonThread = new Thread(new ThreadStart(httpServer.listen));
            DaemonThread.Start();
            txtLog.AppendText(String.Format("[{0}] 서버가 구동 되었습니다.\r\n", DateTime.Now.ToString("HH:mm:ss")));

            IsServerStart = true;
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            try
            {
                if (DaemonThread != null)
                {
                    if (DaemonThread.IsAlive)
                    {
                        httpServer.Stop();
                    }
                }
            }
            catch (Exception)
            {
            }

            txtLog.AppendText(String.Format("[{0}] 서버가 정지 되었습니다.\r\n", DateTime.Now.ToString("HH:mm:ss")));

            IsServerStart = false;
        }
    }
}